@@ -41,7 +41,7 @@ G_BEGIN_DECLS
 /**
  * GstBinFlags:
  * @GST_BIN_FLAG_NO_RESYNC: don't resync a state change when elements are
- *             added or linked in the bin.
+ *             added or linked in the bin (Since 1.0.5)
  * @GST_BIN_FLAG_LAST: the last enum in the series of flags for bins.
  * Derived classes can use this as first value in a list of flags.
  *
@@ -62,7 +62,7 @@ typedef enum {
  * Check if @bin will resync its state change when elements are added and
  * removed.
  *
- * Since: 1.1.1
+ * Since: 1.0.5
  */
 #define GST_BIN_IS_NO_RESYNC(bin)        (GST_OBJECT_FLAG_IS_SET(bin,GST_BIN_FLAG_NO_RESYNC))
 
@@ -198,6 +198,10 @@ GstIterator*	gst_bin_iterate_all_by_interface (GstBin *bin, GType iface);
 gboolean        gst_bin_recalculate_latency      (GstBin * bin);
 
 
+#ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
+G_DEFINE_AUTOPTR_CLEANUP_FUNC(GstBin, gst_object_unref)
+#endif
+
 G_END_DECLS
 
 
