@@ -22,6 +22,8 @@
 #ifndef __GST_QUARK_H__
 #define __GST_QUARK_H__
 
+#include <glib.h>
+
 /* These enums need to match the number and order
  * of strings declared in _quark_table, in gstquark.c */
 typedef enum _GstQuarkId
@@ -196,7 +198,11 @@ typedef enum _GstQuarkId
   GST_QUARK_MESSAGE_STREAM_START = 167,
   GST_QUARK_GROUP_ID = 168,
   GST_QUARK_URI_REDIRECTION = 169,
-  GST_QUARK_MAX = 170
+  GST_QUARK_MESSAGE_DEVICE_ADDED = 170,
+  GST_QUARK_MESSAGE_DEVICE_REMOVED = 171,
+  GST_QUARK_DEVICE = 172,
+  GST_QUARK_URI_REDIRECTION_PERMANENT = 173,
+  GST_QUARK_MAX = 174
 } GstQuarkId;
 
 extern GQuark _priv_gst_quark_table[GST_QUARK_MAX];
