@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <gst/check/gstcheck.h>
@@ -760,16 +760,16 @@ GST_START_TEST (test_add_live)
 
 GST_END_TEST;
 
-static GMutex *blocked_lock;
-static GCond *blocked_cond;
+static GMutex blocked_lock;
+static GCond blocked_cond;
 
 static GstPadProbeReturn
 pad_blocked_cb (GstPad * pad, GstPadProbeInfo * info, gpointer user_data)
 {
-  g_mutex_lock (blocked_lock);
+  g_mutex_lock (&blocked_lock);
   GST_DEBUG ("srcpad blocked: %d, sending signal", info->type);
-  g_cond_signal (blocked_cond);
-  g_mutex_unlock (blocked_lock);
+  g_cond_signal (&blocked_cond);
+  g_mutex_unlock (&blocked_lock);
 
   return GST_PAD_PROBE_OK;
 }
@@ -782,9 +782,6 @@ GST_START_TEST (test_add_live2)
   GstPad *srcpad, *sinkpad;
   gulong id;
 
-  blocked_lock = g_mutex_new ();
-  blocked_cond = g_cond_new ();
-
   pipeline = gst_pipeline_new ("pipeline");
   src = gst_element_factory_make ("fakesrc", "src");
   g_object_set (G_OBJECT (src), "is-live", TRUE, NULL);
@@ -797,7 +794,7 @@ GST_START_TEST (test_add_live2)
   ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
   fail_unless (ret == GST_STATE_CHANGE_ASYNC, "no ASYNC state return");
 
-  g_mutex_lock (blocked_lock);
+  g_mutex_lock (&blocked_lock);
 
   GST_DEBUG ("blocking srcpad");
   /* block source pad */
@@ -817,8 +814,8 @@ GST_START_TEST (test_add_live2)
   gst_bin_add (GST_BIN (pipeline), src);
 
   /* wait for pad blocked, this means the source is now PLAYING. */
-  g_cond_wait (blocked_cond, blocked_lock);
-  g_mutex_unlock (blocked_lock);
+  g_cond_wait (&blocked_cond, &blocked_lock);
+  g_mutex_unlock (&blocked_lock);
 
   GST_DEBUG ("linking pads");
 
@@ -845,8 +842,6 @@ GST_START_TEST (test_add_live2)
   ret = gst_element_set_state (pipeline, GST_STATE_NULL);
   fail_unless (ret == GST_STATE_CHANGE_SUCCESS, "not SUCCESS");
 
-  g_cond_free (blocked_cond);
-  g_mutex_free (blocked_lock);
   gst_object_unref (pipeline);
 }
 
@@ -924,6 +919,7 @@ GST_START_TEST (test_fake_eos)
   GstPad *sinkpad;
   GstFlowReturn res;
   GThread *thread;
+  GstSegment segment;
 
   pipeline = gst_pipeline_new ("pipeline");
 
@@ -937,6 +933,10 @@ GST_START_TEST (test_fake_eos)
   ret = gst_element_set_state (pipeline, GST_STATE_PLAYING);
   fail_unless (ret == GST_STATE_CHANGE_ASYNC, "no ASYNC state return");
 
+  gst_segment_init (&segment, GST_FORMAT_BYTES);
+  gst_pad_send_event (sinkpad, gst_event_new_stream_start ("test"));
+  gst_pad_send_event (sinkpad, gst_event_new_segment (&segment));
+
   /* push buffer of 100 seconds, since it has a timestamp of 0, it should be
    * rendered immediately and the chain function should return immediately */
   buffer = gst_buffer_new_and_alloc (10);
@@ -952,7 +952,8 @@ GST_START_TEST (test_fake_eos)
   /* push EOS, this will block for up to 100 seconds, until the previous
    * buffer has finished. We therefore push it in another thread so we can do
    * something else while it blocks. */
-  thread = g_thread_create ((GThreadFunc) send_eos, sinkpad, TRUE, NULL);
+  thread =
+      g_thread_try_new ("gst-check", (GThreadFunc) send_eos, sinkpad, NULL);
   fail_if (thread == NULL, "no thread");
 
   /* wait a while so that the thread manages to start and push the EOS */
@@ -1074,7 +1075,10 @@ GST_START_TEST (test_async_done)
   /* set bus on element synchronously listen for ASYNC_DONE */
   bus = gst_bus_new ();
   gst_element_set_bus (sink, bus);
-  gst_bus_set_sync_handler (bus, (GstBusSyncHandler) async_done_func, sink);
+  gst_bus_set_sync_handler (bus, (GstBusSyncHandler) async_done_func, sink,
+      NULL);
+
+  gst_pad_send_event (sinkpad, gst_event_new_stream_start ("test"));
 
   /* make newsegment, this sets the position to 10sec when the buffer prerolls */
   GST_DEBUG ("sending segment");
@@ -1095,7 +1099,8 @@ GST_START_TEST (test_async_done)
   /* last buffer, blocks because preroll queue is filled. Start the push in a
    * new thread so that we can check the position */
   GST_DEBUG ("starting thread");
-  thread = g_thread_create ((GThreadFunc) send_buffer, sinkpad, TRUE, NULL);
+  thread =
+      g_thread_try_new ("gst-check", (GThreadFunc) send_buffer, sinkpad, NULL);
   fail_if (thread == NULL, "no thread");
 
   GST_DEBUG ("waiting 1 second");
@@ -1116,14 +1121,16 @@ GST_START_TEST (test_async_done)
 
   /* join the thread. At this point we know the sink processed the last buffer
    * and the position should now be 210 seconds; the time of the last buffer we
-   * pushed */
+   * pushed. The element has no clock or base-time so it only reports the
+   * last seen timestamp of the buffer, it does not know how much of the buffer
+   * is consumed. */
   GST_DEBUG ("joining thread");
   g_thread_join (thread);
 
   gst_element_query_position (sink, GST_FORMAT_TIME, &position);
   GST_DEBUG ("last buffer position %" GST_TIME_FORMAT,
       GST_TIME_ARGS (position));
-  fail_unless (position == 310 * GST_SECOND, "position is wrong");
+  fail_unless (position == 210 * GST_SECOND, "position is wrong");
 
   gst_object_unref (sinkpad);
 
@@ -1183,7 +1190,8 @@ GST_START_TEST (test_async_done_eos)
   /* set bus on element synchronously listen for ASYNC_DONE */
   bus = gst_bus_new ();
   gst_element_set_bus (sink, bus);
-  gst_bus_set_sync_handler (bus, (GstBusSyncHandler) async_done_eos_func, sink);
+  gst_bus_set_sync_handler (bus, (GstBusSyncHandler) async_done_eos_func, sink,
+      NULL);
 
   /* make newsegment, this sets the position to 10sec when the buffer prerolls */
   GST_DEBUG ("sending segment");
@@ -1205,7 +1213,8 @@ GST_START_TEST (test_async_done_eos)
    * position should now be 10 seconds. */
   GST_DEBUG ("pushing EOS");
   GST_DEBUG ("starting thread");
-  thread = g_thread_create ((GThreadFunc) send_eos, sinkpad, TRUE, NULL);
+  thread =
+      g_thread_try_new ("gst-check", (GThreadFunc) send_eos, sinkpad, NULL);
   fail_if (thread == NULL, "no thread");
 
   /* wait for preroll */
@@ -1226,17 +1235,17 @@ GST_START_TEST (test_async_done_eos)
 
 GST_END_TEST;
 
-static GMutex *preroll_lock;
-static GCond *preroll_cond;
+static GMutex preroll_lock;
+static GCond preroll_cond;
 
 static void
 test_async_false_seek_preroll (GstElement * elem, GstBuffer * buf,
     GstPad * pad, gpointer data)
 {
-  g_mutex_lock (preroll_lock);
+  g_mutex_lock (&preroll_lock);
   GST_DEBUG ("Got preroll buffer %p", buf);
-  g_cond_signal (preroll_cond);
-  g_mutex_unlock (preroll_lock);
+  g_cond_signal (&preroll_cond);
+  g_mutex_unlock (&preroll_lock);
 }
 
 static void
@@ -1252,9 +1261,6 @@ GST_START_TEST (test_async_false_seek)
 {
   GstElement *pipeline, *source, *sink;
 
-  preroll_lock = g_mutex_new ();
-  preroll_cond = g_cond_new ();
-
   /* Create gstreamer elements */
   pipeline = gst_pipeline_new ("test-pipeline");
   source = gst_element_factory_make ("fakesrc", "file-source");
@@ -1276,56 +1282,50 @@ GST_START_TEST (test_async_false_seek)
   gst_element_link (source, sink);
 
   GST_DEBUG ("Now pausing");
-  g_mutex_lock (preroll_lock);
+  g_mutex_lock (&preroll_lock);
   gst_element_set_state (pipeline, GST_STATE_PAUSED);
 
   /* wait for preroll */
   GST_DEBUG ("wait for preroll");
-  g_cond_wait (preroll_cond, preroll_lock);
-  g_mutex_unlock (preroll_lock);
+  g_cond_wait (&preroll_cond, &preroll_lock);
+  g_mutex_unlock (&preroll_lock);
 
-  g_mutex_lock (preroll_lock);
+  g_mutex_lock (&preroll_lock);
   GST_DEBUG ("Seeking");
   fail_unless (gst_element_seek (pipeline, 1.0, GST_FORMAT_BYTES,
           GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, 0, GST_SEEK_TYPE_SET, -1));
 
   GST_DEBUG ("wait for new preroll");
   /* this either prerolls or fails */
-  g_cond_wait (preroll_cond, preroll_lock);
-  g_mutex_unlock (preroll_lock);
+  g_cond_wait (&preroll_cond, &preroll_lock);
+  g_mutex_unlock (&preroll_lock);
 
   GST_DEBUG ("bring pipe to state NULL");
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
   GST_DEBUG ("Deleting pipeline");
   gst_object_unref (GST_OBJECT (pipeline));
-
-  g_mutex_free (preroll_lock);
-  g_cond_free (preroll_cond);
 }
 
 GST_END_TEST;
 
-static GMutex *handoff_lock;
-static GCond *handoff_cond;
+static GMutex handoff_lock;
+static GCond handoff_cond;
 
 static void
 test_async_false_seek_in_playing_handoff (GstElement * elem, GstBuffer * buf,
     GstPad * pad, gpointer data)
 {
-  g_mutex_lock (handoff_lock);
+  g_mutex_lock (&handoff_lock);
   GST_DEBUG ("Got handoff buffer %p", buf);
-  g_cond_signal (handoff_cond);
-  g_mutex_unlock (handoff_lock);
+  g_cond_signal (&handoff_cond);
+  g_mutex_unlock (&handoff_lock);
 }
 
 GST_START_TEST (test_async_false_seek_in_playing)
 {
   GstElement *pipeline, *source, *sink;
 
-  handoff_lock = g_mutex_new ();
-  handoff_cond = g_cond_new ();
-
   /* Create gstreamer elements */
   pipeline = gst_pipeline_new ("test-pipeline");
   source = gst_element_factory_make ("fakesrc", "fake-source");
@@ -1346,28 +1346,25 @@ GST_START_TEST (test_async_false_seek_in_playing)
   GST_DEBUG ("Now playing");
   gst_element_set_state (pipeline, GST_STATE_PLAYING);
 
-  g_mutex_lock (handoff_lock);
+  g_mutex_lock (&handoff_lock);
   GST_DEBUG ("wait for handoff buffer");
-  g_cond_wait (handoff_cond, handoff_lock);
-  g_mutex_unlock (handoff_lock);
+  g_cond_wait (&handoff_cond, &handoff_lock);
+  g_mutex_unlock (&handoff_lock);
 
   GST_DEBUG ("Seeking");
   fail_unless (gst_element_seek (source, 1.0, GST_FORMAT_BYTES,
           GST_SEEK_FLAG_FLUSH, GST_SEEK_TYPE_SET, 0, GST_SEEK_TYPE_SET, -1));
 
-  g_mutex_lock (handoff_lock);
+  g_mutex_lock (&handoff_lock);
   GST_DEBUG ("wait for handoff buffer");
-  g_cond_wait (handoff_cond, handoff_lock);
-  g_mutex_unlock (handoff_lock);
+  g_cond_wait (&handoff_cond, &handoff_lock);
+  g_mutex_unlock (&handoff_lock);
 
   GST_DEBUG ("bring pipe to state NULL");
   gst_element_set_state (pipeline, GST_STATE_NULL);
 
   GST_DEBUG ("Deleting pipeline");
   gst_object_unref (GST_OBJECT (pipeline));
-
-  g_mutex_free (handoff_lock);
-  g_cond_free (handoff_cond);
 }
 
 GST_END_TEST;
